/* Copyright (C) 2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

/**
 * @file myx_gc_canvas.h 
 * @brief Generic canvas main class.
 * 
 */

#ifndef __GC_CANVAS_H__
#define __GC_CANVAS_H__

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the MYSQLGENERICCANVAS_EXPORTS
// symbol defined on the command line. This symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// MYSQLGENERICCANVAS_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef MYSQLGENERICCANVAS_EXPORTS
  #define GENERIC_CANVAS_API  __declspec(dllexport)
#else
  #define GENERIC_CANVAS_API  __declspec(dllimport)
#endif

//----------------------------------------------------------------------------------------------------------------------

// An opaque handle to a rendering context. Must be provided by the viewer.
#ifdef _WINDOWS
  typedef HDC GCContext;
#else
  typedef GLXContext GCContext;
#endif // ifdef _WINDOWS

typedef struct tagVIEWPORT
{
  int Left, Top, Width, Height;
} VIEWPORT, * PVIEWPORT;

typedef enum tagGCError
{
    GC_NO_ERROR = 0, 
    GC_ERROR_CANT_OPEN_FILE, 
    GC_XML_PARSE_ERROR,
    GC_XML_NO_VALID_DOCUMENT, 
    GC_XML_EMPTY_DOCUMENT, 
    GC_OBJECT_NOT_FOUND, 
    GC_CANT_READ_FROM_FILE, 
    GC_CHARSET_CONVERSION_ERROR,
    GC_CHARSET_WRONG_CHARSET_SPECIFIED
} GC_ERROR;

//----------------------------------------------------------------------------------------------------------------------

// CGenericCanvas is the main class of the library and is the base for all further functionality (e.g. it creates and maintains the model).
// Instances are created via the exported CreateGenericCanvas function (if called from non C++ languages).
// CGenericCanvas serves as the controller in the model-view-controller situation, which is used here and communicates with 
// the viewer via callbacks.
// The viewer is platform specific and must be implemented individually. It is responsible to create a canvas controller class.
class GENERIC_CANVAS_API CGenericCanvas
{
public: 
  CGenericCanvas(GCContext Context);
private:
  GCContext FContext;
  COLORREF FBackgroundColor;
  VIEWPORT FViewport;
protected:
  void ClearBuffers();
public:
  virtual void __cdecl Release(void);
  virtual void __cdecl SetBackgroundColor(COLORREF NewColor);
  virtual void __cdecl ApplyBackground();
  virtual void __cdecl SetViewport(PVIEWPORT NewViewport);
  virtual void __cdecl ApplyViewport();
  virtual void __cdecl Render();
  virtual GC_ERROR __cdecl AddFiguresFromFile(char* FileName);
};

//----------------------------------------------------------------------------------------------------------------------

// Factory function to create a generic canvas. This function is exported and must be used by the viewer implementations
// to actually create a canvas instance. This is the only way to get hold of a generic canvas instance.
extern "C" GENERIC_CANVAS_API CGenericCanvas* CreateGenericCanvas(GCContext Context);

#endif // __GC_CANVAS_H__
