package com.mysql.grt;

import java.util.ArrayList;
import java.util.List;

public class GrtList implements GrtXml {

	protected List list = new ArrayList();

	protected Object owner = null;

	public String contentType = "";

	public String contentStructName = "";
	
	public GrtList() {
		super();
	}

	public GrtList(Object owner) {
		super();

		this.owner = owner;
	}
	
	public GrtList(Object owner, String contentStructName) {
		super();

		this.owner = owner;
		this.contentType= "dict";
		this.contentStructName= contentStructName;
	}

	public int size() {
		return list.size();
	}

	public GrtList addObject(Object item) {
		//Assign content-type if it has not already been assigned
		if ((size() == 0) && (contentType.equals(""))) {
			if (item.getClass() == Integer.class) {
				contentType = "int";
			} else if (item.getClass() == Double.class) {
				contentType = "real";
			} else if (item.getClass() == String.class) {
				contentType = "string";
			} else if (GrtList.class.isAssignableFrom(item.getClass())) {
				contentType = "list";
			} else {
				contentType = "dict";
			}
		}
		
		list.add(item);

		// if the owner of this object is a GrtObject,
		// notify the change
		if ((owner != null)
				&& (GrtObject.class.isAssignableFrom(owner.getClass())))
			Grt.getInstance().objectChanged((GrtObject) owner);
		
		return this;
	}

	public Object getObject(int index) {
		return list.get(index);
	}

	public void remove(int index) {
		list.remove(index);

		// if the owner of this object is a GrtObject,
		// notify the change
		if ((owner != null)
				&& (GrtObject.class.isAssignableFrom(owner.getClass())))
			Grt.getInstance().objectChanged((GrtObject) owner);
	}

	public int getIndexOfName(String name) {
		for (int i = 0; i < list.size(); i++) {
			Object obj = getObject(i);

			if (GrtObject.class.isAssignableFrom(obj.getClass())) {
				if (((GrtObject) obj).name.compareToIgnoreCase(name) == 0) {
					return i;
				}
			}
		}

		return -1;
	}
	
	public boolean contains(Object item) {
		return list.contains(item);
	}
	
	public String getGrtXml() {
		return getGrtXml("");
	}

	public String getGrtXml(String keyTag) {
		String xml = "<value type=\"list\" content-type=\"" + contentType
				+ "\"";

		if (contentStructName != "") {
			xml = xml + " content-struct-name=\"" + contentStructName + "\"";
		}
		xml = xml + keyTag + ">\n";

		for (int i = 0; i < size(); i++) {
			xml = xml + Grt.getObjectAsXml(getObject(i), "");
		}

		return xml + "</value>\n";
	}

	public String getGrtRefXml(String keyTag) {
		String xml = "<value type=\"list\" option=\"ref\" content-type=\""
				+ contentType + "\"";

		if (contentStructName != "") {
			xml = xml + " content-struct-name=\"" + contentStructName + "\"";
		}
		xml = xml + keyTag + ">\n";

		for (int i = 0; i < size(); i++) {
			Object obj = getObject(i);

			if (GrtObject.class.isAssignableFrom(obj.getClass())) {
				xml = xml + "<value type=\"string\" option=\"ref\">"
						+ ((GrtObject) obj).get_id() + "</value>\n";
			}
		}

		return xml + "</value>\n";
	}
}