package com.mysql.grt.modules;

import com.mysql.grt.Grt;
import com.mysql.grt.GrtHashMap;
import com.mysql.grt.GrtList;
import com.mysql.grt.GrtTest;

import junit.framework.TestCase;
import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * Test of the Grt Base module class
 * 
 * @author Mike
 * @version 1.0, 11/29/04
 */
public class GrtBaseTest extends TestCase {
	
	public GrtBaseTest(String name) {
		super(name);
	}
	
	/**
	 * main function so the test can be executed
	 */
	public static void main(String[] args) {
		junit.textui.TestRunner.run(GrtTest.class);
	}
	
	/**
	 * The suite function builds the testsuite
	 */
	public static Test suite() {
		TestSuite suite = new TestSuite();
		suite.addTest(new GrtBaseTest("testCallModuleFunctionHelloWorld"));
		suite.addTest(new GrtBaseTest("testCallModuleFunctionUpperCase"));
		suite.addTest(new GrtBaseTest("testCallModuleFunctionUpperCaseWithList"));
		suite.addTest(new GrtBaseTest("testGetListSize"));
		suite.addTest(new GrtBaseTest("testConcatStrings"));
		suite.addTest(new GrtBaseTest("testGetMessages"));
		return suite;
	}

	/**
	 * Tests the callModuleFunction function by calling helloWorld()
	 */
	public void testCallModuleFunctionHelloWorld() {
		String xml = Grt.callModuleFunction(GrtBase.class, "helloWorld", "",
				"");

		GrtHashMap result = (GrtHashMap) Grt.getObjectsFromGrtXml(xml);

		Assert.assertNull(result.getObject("error"));
		Assert.assertEquals("Hello World!", result.getObject("value"));
	}
	
	/**
	 * Tests the callModuleFunction function by calling upperCase() with a
	 * single object as parameter
	 */
	public void testCallModuleFunctionUpperCase() {
		String testStr = "This is a test of the upperCase function.";
		String testParam = Grt.prepareGrtXml(Grt.getObjectAsXml(testStr));
		String xml = Grt.callModuleFunction(GrtBase.class, "upperCase",
				"(Ljava/lang/String;)", testParam);

		GrtHashMap result = (GrtHashMap) Grt.getObjectsFromGrtXml(xml);

		Assert.assertNull(result.getObject("error"));
		Assert.assertEquals(result.getObject("value"), testStr.toUpperCase());
	}

	/**
	 * Tests the callModuleFunction function by calling upperCase() with a
	 * GrtList as parameter
	 */
	public void testCallModuleFunctionUpperCaseWithList() {
		String testStr = "This is a test of the upperCase function.";
		String paramXml = Grt.prepareGrtXml(Grt.getObjectAsXml(new GrtList()
				.addObject(testStr)));
		String xml = Grt.callModuleFunction(GrtBase.class, "upperCase",
				"Ljava/lang/String;", paramXml);

		GrtHashMap result = (GrtHashMap) Grt.getObjectsFromGrtXml(xml);

		Assert.assertNull(result.getObject("error"));
		Assert.assertEquals(result.getObject("value"), testStr.toUpperCase());
	}

	/**
	 * Tests the callModuleFunction function by calling getListSize()
	 */
	public void testGetListSize() {
		String paramXml = Grt.prepareGrtXml(Grt.getObjectAsXml(new GrtList()
				.addObject(new GrtList().addObject("1").addObject("2")
						.addObject("3"))));
		String xml = Grt.callModuleFunction(GrtBase.class, "getListSize",
				"Lcom/mysql/grt/GrtList;", paramXml);

		GrtHashMap result = (GrtHashMap) Grt.getObjectsFromGrtXml(xml);

		int listSize = ((Integer) result.getObject("value")).intValue();

		Assert.assertNull(result.getObject("error"));
		Assert.assertEquals(listSize, 3);
	}
	
	/**
	 * Tests the callModuleFunction function by calling concatStrings()
	 */
	public void testConcatStrings() {
		String s1 = "Hello";
		String s2 = "World";
		String paramXml = Grt.prepareGrtXml(Grt.getObjectAsXml(new GrtList()
				.addObject(s1).addObject(s2)));
		String xml = Grt.callModuleFunction(GrtBase.class, "concatStrings",
				"Ljava/lang/String;Ljava/lang/String;", paramXml);

		GrtHashMap result = (GrtHashMap) Grt.getObjectsFromGrtXml(xml);

		Assert.assertNull(result.getObject("error"));
		Assert.assertEquals(result.getObject("value"), s1 + s2);
	}
	
	public void testGetMessages() {
		String xml = Grt.callModuleFunction(GrtBase.class, "getMessages",
				"", "");
		
		Assert.assertNotNull(xml);
		
		//System.out.println(xml);
	}
}
